
const setupClickHandler = require('../lib/setupClickHandler');
const createInjectableUtils = require('../lib/createInjectableUtils');
const injectJQuery = require('../lib/injectJQuery');

function runWithInjectables(browser, constructScraper, injectables) {
  // The following function will bootstrap the scraper, and the injectables
  // inside the browser context. It does not run in the same context as this
  // function, so you will not be able to share variables etc.

  const bootstrapInBrowser = (constructScraperStr, utilsStr) => {
    async function main() {
      const utils = eval(utilsStr); // eslint-disable-line no-eval

      const fnFromString = (fnStr) => {
        const [, args, body] = fnStr.match(/^function[^(]*\(([^)]*)\) {((?:.|\s)*)}$/);
        return new Function(args.split(','), body); // eslint-disable-line no-new-func
      };

      const scraper = fnFromString(constructScraperStr)();

      try {
        await scraper[utils.queueItem.method](utils); // eslint-disable-line no-undef
        return {
          queue: utils.queue.getQueue(),
          data: utils.data.getData(),
          onRedirect: utils.onRedirect.redirectMethod,
          finalUrl: window.location.href,
        };
      } catch (error) {
        error.from = 'user function';
        throw error;
      }
    }

    return main();
  };

  return browser.evaluate(
    bootstrapInBrowser,
    constructScraper.toString(),
    injectables.build(),
  );
}


module.exports = async function scrapeBrowser({
  queueItem,
  browser,
  scraperStr,
  takeScreenshot,
  log,
}) {
  log.info({ url: queueItem.url }, 'navigate');
  await browser.goto(queueItem.url);

  log.trace('inject tools');
  await injectJQuery(browser);
  await setupClickHandler(browser);

  log.info({ method: queueItem.method }, 'run method');

  let data = await runWithInjectables(
    browser,
    scraperStr,
    createInjectableUtils(queueItem),
  );

  if (data.onRedirect) {
    log.info('redirected, wait for new page');
    await browser.waitForNavigation();
    // eslint-disable-next-line no-param-reassign
    queueItem.method = data.onRedirect;
    log.info({ method: queueItem.method }, 'run method');
    data = await runWithInjectables(browser, scraperStr, createInjectableUtils(queueItem));
  }

  if (takeScreenshot) {
    log.info('take screenshot');
    const screenshot = await browser.screenshot({ fullPage: false });
    return { ...data, screenshot };
  }

  return data;
};
