const createPool = require('./lib/browser');
const scrapeBrowser = require('./lib/scrapeBrowser');

let futurePool;

module.exports = async function processBrowserLite({
  queueItem, scraper, takeScreenshot = false, log,
}) {
  if (futurePool === undefined) {
    futurePool = createPool({ min: 0, max: 20 });
  }
  const currentPool = futurePool;
  const pool = await currentPool;

  log.debug('acquire browser window');
  const browser = await pool.acquire();

  const emptyData = {
    queue: [],
    data: undefined,
    finalUrl: queueItem.url,
  };

  try {
    return await scrapeBrowser({
      queueItem,
      browser,
      scraperStr: typeof scraper === 'string' ? scraper : scraper.construct,
      takeScreenshot,
      log,
    });
  } catch (err) {
    if (err.message.match(/Execution context was destroyed/)) {
      return emptyData;
    }

    if (err.message.match(/Promise was collected/)) {
      console.log('Promise was collected?', err);
      throw err;
    }

    if (err.message.match(/^Navigation Timeout Exceeded/)) {
      // @todo Need to manage this so it doesn't end in endless cycle
      console.log('Retrying because of navigation error');
      return processBrowserLite({ queueItem, scraper });
    }

    if (
      err.message.match(/^Protocol error \([^)]*\):/)
    ) {
      console.log('Probable browser crash', err.message);
      if (currentPool === futurePool) {
        console.log('create new pool');
        futurePool = createPool({ min: 0, max: 20 });
        console.log('kill and create a new pool');
        pool.kill();
      } else {
        console.log('pool already changed, just retry');
      }

      return processBrowserLite({ queueItem, scraper });
    }

    throw err;
  } finally {
    await pool.release(browser);
  }
};
